import solitaire

def entete(numero):
    ligne = '=== "P'+str(numero)+'"\n\n'
    return ligne

def enonce(numero):
    s = '\t??? question "Enoncé et solution de E.Lucas"\n\n'
    s += f'\t\t![p{numero}](../assets/images/solitaire/p{numero:02}.png)\n\n'
    return s

def charger(pb, numero):
    s = "\t### Charger le problème et afficher l'objectif\n\n"

    s += '\t```python\n'
    s += f'\t>>> p{numero} = Solitaire("EL{numero:02}")\n'
    s += f'\t>>> p{numero}.goal()\n'
    s += '\t```\n\n'

    s += '\t```\n'
    s += pb.goal(1)
    s += '\n\t```\n'
    return s

def solution(pb, numero):
    sol = solitaire.LUCAS_SOLUTIONS[numero]
    s = '\t### Tester la solution de E. Lucas\n\n'

    s += '\t```python\n'
    s += f'\t>>> p{numero}.solution({sol})\n'
    s += f'\t{pb.solution(sol)}\n'
    s += f'\t>>> p{numero}.coups()\n'
    s += f'\t{pb.coups()}\n'
    s += '\t```\n\n'
    return s

def notre_solution(pb, numero, limit=solitaire.INFINI):
    rep = pb.solve(limit=limit)
    coups = pb.coups()
    explored = pb.explored

    s = '\t### Notre solution\n\n'

    s += '\t```python\n'
    s += f"\t>>> p{numero}b = Solitaire('EL{numero:02}')\n"
    s += f'\t>>> p{numero}b.solve()\n'
    s += f'\t{rep}\n'
    s += f'\t>>> p{numero}b.coups()\n'
    s += f'\t{coups}\n'
    s += f'\t>>> p{numero}b.explored\n'
    s += f'\t{explored}\n'
    s += '\t```\n\n'
    return s

def trace(p, numero):
    str_trace = p.trace(2)

    s = '\t??? done "La trace complète"\n\n'

    s += '\t\t```python\n'
    s += f'\t\t>>> p{numero}.trace()\n'
    s += '\t\t```\n\n'

    s += '\t\t```\n'
    s += str_trace 
    s += '\n'
    s += '\t\t```\n'
    return s


def probleme(numero, notre_sol=False, limit=solitaire.INFINI):
    racine = f'EL{numero:02}'
    p = solitaire.Solitaire(racine)
    pb = solitaire.Solitaire(racine)
    s = entete(numero)
    s += enonce(numero)
    s += charger(p, numero)
    s += solution(p, numero)
    if notre_sol:
        s += notre_solution(pb, numero, limit)
        s += trace(pb, numero)
    return s

def part1():
    for i in range(1, 7):
        with open(f'../docs/includes/pb{i:02}.md', 'w', encoding='utf-8') as output:
            output.write(probleme(i, True))

def part2():
    for i in range(7, 13):
        with open(f'../docs/includes/pb{i:02}.md', 'w', encoding='utf-8') as output:
            output.write(probleme(i))

def part3():
    for i in range(13, 31):
        with open(f'../docs/includes/pb{i:02}.md', 'w', encoding='utf-8') as output:
            output.write(probleme(i))